from .unit.testregister import *
from .unit.testmanagers import *
from .unit.testmodels import *
from .unit.testadmin import *
from .unit.testdiff import *
from .unit.testforms import *
from .unit.testmoderator import *
from .regression import *
from .acceptance.testexclude import *
from .acceptance.testauto_discover import *
